"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const vscode_1 = require("vscode");
const interfaces_1 = require("../interfaces");
const resources_1 = require("../resources");
const utils_1 = require("../utils");
class default_1 {
    provideCompletionItems(document, position, token, context) {
        let char = context.triggerCharacter;
        switch (char) {
            // Component
            case '<':
                return this.genComponent(char);
            // Property
            case '"':
            case ' ':
            case '(':
            case '[':
                return this.genProperties(document, position, char);
            default:
                return [];
        }
    }
    genComponent(char) {
        return resources_1.RESOURCES.map(i => this.renderCompletionItem(char, i, false));
    }
    genDirective() {
        return resources_1.RESOURCES.filter(w => w.type === 'directive').map(i => this.renderCompletionItem('', i, true));
    }
    renderCompletionItem(char, i, isClean) {
        const item = new vscode_1.CompletionItem(i.selector, vscode_1.CompletionItemKind.Property);
        item.command = this.triggerHideSuggestCommand;
        item.documentation = i._doc;
        if (isClean) {
            return item;
        }
        // `<` 非单词部分需要提前移除触发词
        if (char === '<') {
            item.insertText = new vscode_1.SnippetString(i.snippet.substr(1));
        }
        else {
            item.insertText = new vscode_1.SnippetString(i.snippet);
        }
        return item;
    }
    genProperties(document, position, triggerCharacter) {
        const tag = utils_1.getTag(document, position);
        if (tag == null)
            return this.genDirective();
        const directive = resources_1.getDirective(tag);
        if (directive == null)
            return this.genDirective();
        if (tag.isOnAttrValue) {
            const attr = tag.attributes[tag.attrName];
            if (attr.value.trim() === '') {
                const property = directive.properties.find(w => w.name === attr.name);
                if (property && property.typeDefinition) {
                    let range = document.getWordRangeAtPosition(position, /["]\s*["]/);
                    if (range) {
                        range = new vscode_1.Range(new vscode_1.Position(range.start.line, range.start.character + 1), new vscode_1.Position(range.end.line, range.end.character - 1));
                    }
                    let definitionList = property.typeDefinition;
                    if (property.inputType === interfaces_1.InputAttrType.Complex) {
                        definitionList = this.getComplexDefinition(tag, definitionList);
                    }
                    return definitionList.map((i) => {
                        const item = new vscode_1.CompletionItem(i.value, vscode_1.CompletionItemKind.EnumMember);
                        item.documentation = new vscode_1.MarkdownString(i.label);
                        item.range = range;
                        return item;
                    });
                }
            }
            else if (attr.value.startsWith('{')) {
                // 复杂类型
                const property = directive.properties.find(w => w.name === attr.name);
                if (property && property.complexType) {
                    return directive.types[property.complexType].map(i => {
                        const item = new vscode_1.CompletionItem(i.name, vscode_1.CompletionItemKind.Value);
                        item.documentation = i._doc;
                        return item;
                    });
                }
            }
            return [];
        }
        const existsAttrList = Object.keys(tag.attributes);
        if (triggerCharacter === '[') {
            let range = document.getWordRangeAtPosition(position, /\[\s*\]/);
            if (range) {
                range = new vscode_1.Range(new vscode_1.Position(range.start.line, range.start.character + 1), new vscode_1.Position(range.end.line, range.end.character));
            }
            return directive.properties
                .filter(w => existsAttrList.indexOf(w.name) === -1)
                .filter(w => w.inputType === interfaces_1.InputAttrType.Input || w.inputType === interfaces_1.InputAttrType.InputOutput)
                .map(i => this.renderAttrCompletionItem(tag, i, 'property', range));
        }
        if (triggerCharacter === '(') {
            let range = document.getWordRangeAtPosition(position, /\(\s*\)/);
            if (range) {
                range = new vscode_1.Range(new vscode_1.Position(range.start.line, range.start.character + 1), new vscode_1.Position(range.end.line, range.end.character));
            }
            return directive.properties
                .filter(w => existsAttrList.indexOf(w.name) === -1)
                .filter(w => w.inputType === interfaces_1.InputAttrType.Output || w.inputType === interfaces_1.InputAttrType.InputOutput)
                .map(i => this.renderAttrCompletionItem(tag, i, 'event', range));
        }
        return directive.properties
            .filter(w => existsAttrList.indexOf(w.name) === -1)
            .map(i => this.renderAttrCompletionItem(tag, i, '', null));
    }
    renderAttrCompletionItem(tag, property, ngBindingType, range) {
        const item = new vscode_1.CompletionItem(property.name, vscode_1.CompletionItemKind.Field);
        item.documentation = property._doc;
        if (range) {
            item.range = range;
        }
        // 优化复杂类型
        if (property.inputType === interfaces_1.InputAttrType.Complex) {
            const complexDefinitionList = this.getComplexDefinition(tag, property.typeDefinition)
                .map(i => i.value)
                .join(',');
            item.insertText = new vscode_1.SnippetString(`${property.name}="\${1|${complexDefinitionList}|}"$0`);
            return item;
        }
        let snippet = '';
        if (property.snippet) {
            snippet = property.snippet;
        }
        else {
            switch (ngBindingType) {
                // 属性绑定
                case 'property':
                    snippet = `${property.name}]=${this.genDefaultPropertyValueSnippet(property)}`;
                    break;
                // 事件绑定
                case 'event':
                    snippet = `${property.name})=${this.genDefaultPropertyValueSnippet(property)}`;
                    break;
                // 默认绑定
                default:
                    snippet = this.genDefaultPropertySnippet(property);
                    break;
            }
        }
        if (!snippet) {
            snippet = `${property.name}="\${1:${property.pureDefault ? property.pureDefault : property.pureName}}"$0`;
        }
        item.insertText = new vscode_1.SnippetString(snippet);
        return item;
    }
    genDefaultPropertySnippet(property) {
        if (property.inputType === interfaces_1.InputAttrType.InputOutput) {
            return `[(${property.name})]=${this.genDefaultPropertyValueSnippet(property)}`;
        }
        // 对于指定强制性类型，不再给予任何建议，始终认为属于需要绑定变量名
        if (property.forceInputType != null) {
            switch (property.forceInputType) {
                case interfaces_1.InputAttrType.InputOutput:
                    return `[(${property.name})]="\${1:${property.pureName}}"$0`;
                case interfaces_1.InputAttrType.Input:
                    return `[${property.name}]="\${1:${property.pureName}}"$0`;
                case interfaces_1.InputAttrType.Output:
                    return `(${property.name})="\${1:${property.pureName}}(\${2:\\$event})"$0`;
            }
        }
        switch (property.type) {
            case 'string':
            case 'Enum':
                if (property.typeDefinitionSnippetStr.length > 0) {
                    return `${property.name}=${this.genDefaultPropertyValueSnippet(property)}`;
                }
                break;
            case 'boolean':
                return `${property.name}=${this.genDefaultPropertyValueSnippet(property)}`;
            case 'any':
            case 'object':
            case 'Date':
            case 'Array':
            case 'function':
            case 'TemplateRef':
                return `[${property.name}]=${this.genDefaultPropertyValueSnippet(property)}`;
            case 'EventEmitter':
                return `(${property.name})=${this.genDefaultPropertyValueSnippet(property)}`;
        }
    }
    genDefaultPropertyValueSnippet(property) {
        switch (property.type) {
            case 'boolean':
                return `"\${1|true,false${resources_1.CONFIG.isAlain ? ',http.loading' : ''}|}"$0`;
            case 'TemplateRef':
                return property.default ? `"\${1:${property.default}}"$0` : `"\${1:${property.pureName}}Tpl"$0`;
            case 'Array':
            case 'function':
                return `"\${1:${property.pureName}}"$0`;
            case 'EventEmitter':
                if (~property.typeRaw.indexOf('boolean')) {
                    return `"\${1:${property.pureName}}(\${2|$event,true,false|})"$0`;
                }
                return `"\${1:${property.pureName}}(\${2:\\$event})"$0`;
            default:
                if (property.typeDefinitionSnippetStr.length > 0) {
                    return `"\${1|${property.typeDefinitionSnippetStr}|}"$0`;
                }
                else {
                    return `"\${1:${property.pureName}}"$0`;
                }
        }
    }
    getComplexDefinition(tag, definitionComplex) {
        const expectAttr = tag.attributes[definitionComplex.conditionField];
        const expectValue = expectAttr ? expectAttr.value : '';
        let expectIndex = definitionComplex.list.findIndex(w => w.conditionValue == expectValue);
        return definitionComplex.list[expectIndex < 0 ? 0 : expectIndex].values;
    }
    get triggerShowSuggestCommand() {
        return {
            command: 'editor.action.triggerSuggest',
            title: 'showSuggestWidget',
        };
    }
    get triggerHideSuggestCommand() {
        return {
            command: 'hideSuggestWidget',
            title: 'hideSuggestWidget',
        };
    }
}
exports.default = default_1;
//# sourceMappingURL=completion-provider.js.map