"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const vscode_1 = require("vscode");
const interfaces_1 = require("./interfaces");
const resources_1 = require("./resources");
function getOffsetText(document, position) {
    const text = document.getText();
    const offset = document.offsetAt(position);
    const maxEnd = text.length;
    let start = offset - 1;
    let end = offset;
    let enterCount = 0;
    while (start >= 0 && text[start] !== '<') {
        if (text[start] === '\n')
            ++enterCount;
        start--;
    }
    while (end <= maxEnd && text[end] !== '>') {
        end++;
    }
    const cursor = offset - start - enterCount;
    return {
        start,
        end,
        cursor,
        text: text.slice(start, end).replace(/\n/g, ''),
    };
}
function getTag(doc, pos, includeAttr = true) {
    const offsetText = getOffsetText(doc, pos); // doc.lineAt(pos.line).text;
    const replacer = (char) => (raw) => char.repeat(raw.length);
    const pureLine = offsetText.text.replace(/\{\{[^\}]*?\}\}/g, replacer('^'));
    const attrFlagLine = pureLine.replace(/("[^"]*"|'[^']*')/g, replacer('%'));
    let tag;
    let attrstr = '';
    pureLine.replace(/<([\-\w]+)(\s+[^>]*)?/g, (raw, name, attr, index) => {
        attrstr = offsetText.text.substr(index + raw.indexOf(attr));
        if (!tag && index <= offsetText.cursor && index + raw.length >= offsetText.cursor) {
            let range = doc.getWordRangeAtPosition(pos, /\b[\w-:.]+\b/);
            let posWord = '';
            let attrName = '';
            if (range) {
                posWord = doc.getText(range);
            }
            let isOnTagName = offsetText.cursor <= index + name.length + 1;
            // fix directive
            if (!isOnTagName && resources_1.isComponent(posWord)) {
                isOnTagName = true;
            }
            let isOnAttrValue = attrFlagLine[offsetText.cursor] === '%';
            if (isOnAttrValue) {
                attrName = getAttrName(attrFlagLine.substring(0, offsetText.cursor));
            }
            let isOnAttrName = !isOnTagName && !isOnAttrValue && !!posWord;
            tag = {
                name,
                word: posWord,
                isOnTagName,
                isOnAttrName,
                isOnAttrValue,
                attrName,
            };
        }
        return raw;
    });
    if (tag && includeAttr) {
        tag.attributes = getAttrs(attrstr);
    }
    return tag;
}
exports.getTag = getTag;
function getAttrs(text) {
    const re = /((?:\[|\(|\[\(|#)?[\-\w]+(?:\)\]|\]|\))?)(?:="([^"]+)")?/g;
    const res = {};
    let match;
    while ((match = re.exec(text))) {
        const item = pureAttr(match[1], match[2] || '');
        res[item.name] = item;
    }
    return res;
}
exports.getAttrs = getAttrs;
function getAttrName(str) {
    if (/\s([\[\]\(\)\-\w]+)=%*$/.test(str)) {
        return pureAttrName(RegExp.$1 || '').name;
    }
    return '';
}
exports.getAttrName = getAttrName;
function pureAttr(name, value) {
    let valueType = interfaces_1.TagAttrValueType.String;
    if (~value.indexOf(`'`) && ~name.indexOf('[')) {
        valueType = interfaces_1.TagAttrValueType.PropertyBinding;
    }
    else if (value === 'true' || value === 'false') {
        valueType = interfaces_1.TagAttrValueType.Boolean;
    }
    else if (/^[.0-9]+$/g.test(value)) {
        valueType = interfaces_1.TagAttrValueType.Number;
    }
    return Object.assign({}, pureAttrName(name), { value, valueType });
}
exports.pureAttr = pureAttr;
function pureAttrName(name) {
    let type;
    if (~name.indexOf('[(')) {
        type = interfaces_1.InputAttrType.InputOutput;
        name = name.substr(2, name.length - 4);
    }
    else if (~name.indexOf('[')) {
        name = name.substr(1, name.length - 2);
    }
    else if (~name.indexOf('(')) {
        type = interfaces_1.InputAttrType.Output;
        name = name.substr(1, name.length - 2);
    }
    else if (~name.indexOf('#')) {
        type = interfaces_1.InputAttrType.Template;
        name = name.substr(1);
    }
    return { type, name };
}
exports.pureAttrName = pureAttrName;
/**
 * Get pre-character in current line
 */
function typingPreChart(document, position, count = 1) {
    return document.getText(new vscode_1.Range(new vscode_1.Position(position.line, position.character - count), position));
}
exports.typingPreChart = typingPreChart;
//# sourceMappingURL=utils.js.map